
#include "frame.h"
#include "vector.h"
#include "povproto.h"
#include "camera.h"
#include "matrices.h"
#include "normal.h"

CAMERA::CAMERA()
{
  
  Make_Vector(Location,    0.0,  0.0, 0.0);
  Make_Vector(Direction,   0.0,  0.0, 1.0);
  Make_Vector(Up,          0.0,  1.0, 0.0);
  Make_Vector(Right,       1.33, 0.0, 0.0);
  Make_Vector(Sky,         0.0,  1.0, 0.0);
  Make_Vector(Look_At,     0.0,  0.0, 1.0);
  Make_Vector(Focal_Point, 0.0,  0.0, 1.0);

  /* Init focal blur stuff (not used by default). */
  Blur_Samples   = 0;
  Confidence     = 0.9;
  Variance       = 1.0 / 10000.0;
  Aperture       = 0.0;
  Focal_Distance = -1.0;

  /* Set default camera type and viewing angle. [DB 7/94] */
  Type = PERSPECTIVE_CAMERA;
  Angle = 90.0;

  /* Default view angle for spherical camera. [MH 6/99] */
  H_Angle = 360;
  V_Angle = 180;

  /* Do not perturb primary rays by default. [DB 7/94] */
  Tnormal = NULL;

  Trans = Create_Transform();

}
CAMERA::~CAMERA()
{
  delete Tnormal;
    Destroy_Transform(Trans);
}
void CAMERA::Translate( VECTOR Vector)
{
  VAddEq(Location, Vector);

}
void CAMERA:: Rotate( VECTOR Vector)
{
  TRANSFORM Trans;
  
  Compute_Rotation_Transform(&Trans, Vector);
  
  Transform( &Trans);

}
void CAMERA::Scale( VECTOR Vector)
{
  TRANSFORM Trans;
  
  Compute_Scaling_Transform(&Trans, Vector);
  
  Transform(&Trans);
}
void CAMERA::Transform ( TRANSFORM *Transf)
{
  MTransPoint(Location, Location, Transf);
  MTransDirection(Direction, Direction, Transf);
  MTransDirection(Up, Up, Transf);
  MTransDirection(Right, Right, Transf);

}
CAMERA* CAMERA::GetCopy()
{
 CAMERA *New;

    New = new CAMERA();

   // Destroy_Tnormal(New->Tnormal);
	delete New->Tnormal;
    Destroy_Transform(New->Trans);

    *New = *this;
    New->Tnormal = NULL; // clear in case the copy fails
    if(this->Tnormal != NULL)
       New->Tnormal = (TNORMAL*)this->Tnormal->GetCopy();//new TNORMAL(Old->Tnormal);//Copy_Tnormal(Old->Tnormal);

    New->Trans = NULL; // clear in case the copy fails
    if(this->Trans != NULL)
       New->Trans = Copy_Transform(this->Trans);
 

  return (New);
}

