
#ifndef CAMERA_H
#define CAMERA_H



/*****************************************************************************
* Global preprocessor defines
******************************************************************************/




/* Available camera types. [DB 8/94] */

#define PERSPECTIVE_CAMERA      1
#define ORTHOGRAPHIC_CAMERA     2
#define FISHEYE_CAMERA          3
#define ULTRA_WIDE_ANGLE_CAMERA 4
#define OMNIMAX_CAMERA          5
#define PANORAMIC_CAMERA        6
#define CYL_1_CAMERA            7
#define CYL_2_CAMERA            8
#define CYL_3_CAMERA            9
#define CYL_4_CAMERA           10
#define SPHERICAL_CAMERA       11


/*****************************************************************************
* Global typedefs
******************************************************************************/

//typedef struct Camera_Struct CAMERA;

class CAMERA
{
public:
  VECTOR Location;
  VECTOR Direction;
  VECTOR Up;
  VECTOR Right;
  VECTOR Sky;
  VECTOR Look_At;                /* Used only to record the user's preference */
  VECTOR Focal_Point;            /* Used only to record the user's preference */
  DBL Focal_Distance, Aperture;  /* ARE 9/92 for focal blur.                  */
  int Blur_Samples;              /* ARE 9/92 for focal blur.                  */
  DBL Confidence;                /* Probability for confidence test.          */
  DBL Variance;                  /* Max. variance for confidence test.        */
  int Type;                      /* Camera type.                              */
  DBL Angle;                     /* Viewing angle.                            */
  DBL H_Angle;                   /* Spherical horizontal viewing angle        */
  DBL V_Angle;                   /* Spherical verticle viewing angle          */
  TNORMAL *Tnormal;              /* Primary ray pertubation.                  */
  TRANSFORM *Trans;              /* Used only to record the user's input      */

virtual  void Translate( VECTOR Vector);
virtual  void Rotate( VECTOR Vector);
virtual  void Scale( VECTOR Vector);
virtual  void Transform ( TRANSFORM *Transf);
virtual  CAMERA *GetCopy();
CAMERA();
~CAMERA();
};



/*****************************************************************************
* Global variables
******************************************************************************/




/*****************************************************************************
* Global functions
******************************************************************************/
/*
void Translate_Camera (CAMERA *Cm, VECTOR Vector);
void Rotate_Camera (CAMERA *Cm, VECTOR Vector);
void Scale_Camera (CAMERA *Cm, VECTOR Vector);
void Transform_Camera (CAMERA *Cm, TRANSFORM *Trans);
CAMERA *Copy_Camera (CAMERA *Old);
CAMERA *Create_Camera (void);
void Destroy_Camera (CAMERA *Cm);
*/

#endif
