#include "frame.h"
#include "povproto.h"           /* Error() */

#include "povray.h"
//#include "povms.h"
#include "helpers.h"
//#include "userio.h"


/*****************/
#include <ctype.h>
#include <math.h>
#include <algorithm>

#include "frame.h"
#include "vector.h"
#include "povproto.h"
//#include "parse.h"
//#include "parstxtr.h"
//#include "parsestr.h"
#include "atmosph.h"
#include "bezier.h"   
//#include "blob.h"     
#include "boxes.h"
//#include "bsphere.h"
#include "colour.h"
#include "cones.h"    
#include "csg.h"
#include "discs.h"
//#include "express.h"  
#include "fractal.h"
//#include "gif.h"      
#include "hfield.h"
//#include "iff.h"      
#include "image.h"    
#include "interior.h"    
//#include "isosurf.h" 
#include "lathe.h"    
#include "lightgrp.h"
#include "matrices.h"
#include "mesh.h"
#include "normal.h"
#include "objects.h"
#include "octree.h"
//#include "pgm.h"      
#include "pigment.h"
#include "planes.h"
#include "polygon.h"
#include "polysolv.h"
#include "poly.h"
#include "povray.h"   
//#include "ppm.h"      
#include "prism.h"    
#include "quadrics.h" 
#include "radiosit.h"      
#include "render.h"   
#include "sor.h"      
#include "spheres.h"  
#include "sphsweep.h"   /* Sphere sweep support */
#include "splines.h"
#include "super.h"
//#include "targa.h"    
#include "texture.h"  
//#include "tokenize.h" 
#include "torus.h"
#include "triangle.h" 
#include "truetype.h" 
#include "photons.h"
//#include "fpmetric.h"
#include "file_pov.h"
/**************************/
void MAError (char *str,size_t size)
{
  Error ("Out of memory.  Cannot allocate %ld bytes for %s.",size,str);
}
void Link_Textures (TEXTURE **Old_Textures, TEXTURE *New_Textures)
{
	TEXTURE *Layer;
	
	if (New_Textures == NULL)
		return;
	
	if ((*Old_Textures) != NULL)
	{
		if ((*Old_Textures)->Type != PLAIN_PATTERN) 
		{
			Error("Cannot layer over a patterned texture.");
		}
	}
	for (Layer = New_Textures ;
	Layer->Next != NULL ;
	Layer = (TEXTURE *)Layer->Next)
	{
		/* NK layers - 1999 June 10 - for backwards compatiblity with layered textures */
		if(opts.Language_Version<=310)
			Convert_Filter_To_Transmit(Layer->Pigment);
	}
	
	/* NK layers - 1999 Nov 16 - for backwards compatiblity with layered textures */
	if ((opts.Language_Version<=310) && (*Old_Textures!=NULL))
		Convert_Filter_To_Transmit(Layer->Pigment);
	
	Layer->Next = (TPATTERN *)*Old_Textures;
	*Old_Textures = New_Textures;
	
	if ((New_Textures->Type != PLAIN_PATTERN) && (New_Textures->Next != NULL))
	{
		Error("Cannot layer a patterned texture over another.");
	}
}

/*****************************************************************************
*
* FUNCTION
*
*   Link_To_Frame
*
* INPUT
*
*   Object - Pointer to object
*   
* OUTPUT
*
*   Object
*   
* RETURNS
*   
* AUTHOR
*
*   POV-Ray Team
*   
* DESCRIPTION
*
*   -
*
* CHANGES
*
*   Sep 1994 : Added optional splitting of bounded unions if children are
*              finite. Added removing of unnecessary bounding. [DB]
*
******************************************************************************/

void Link_To_Frame(OBJECT *Object)
{
  int finite;
  DBL Volume;
  OBJECT *This_Sib, *Next_Sib;

  if (Object == NULL)           /* patches a memory addressing error jdm mar/95 */
    return;

  /* Remove bounding object if object is cheap to intersect. [DB 8/94]  */

  if ((opts.Options & REMOVE_BOUNDS) && (Object->Bound != NULL))
  {
    if ((Object->oType != CSG_UNION_TYPE)        &&
        (Object->oType != CSG_INTERSECTION_TYPE) &&
        (Object->oType != CSG_MERGE_TYPE)        &&
        (Object->oType != POLY_TYPE)             &&
        (Object->oType != TTF_TYPE)			   &&
		(Object->oType != QUADRIC_TYPE ||
			((QUADRIC *)Object)->Automatic_Bounds))
    {
      /* Destroy only, if bounding object is not used as clipping object. */

      if (Object->Bound != Object->Clip)
      {
        Destroy_Object(Object->Bound);
      }

      Object->Bound = NULL;

      Warning(0, "Unnecessary bounding object removed.");
    }
  }

  /*
   * [CJC 8/01]
   *
   * if all children of a union have the no_shadow flag, then the union should
   * have it as well.
   */
  if (Object->oType == CSG_UNION_TYPE)
  {
    for (This_Sib = ((CSG * ) Object)->Children ; This_Sib != NULL ; This_Sib = This_Sib->Sibling)
      if (This_Sib->oType != LIGHT_TYPE && !Test_Flag (This_Sib, NO_SHADOW_FLAG))
        break ;
    if (This_Sib == NULL)
      Set_Flag (Object, NO_SHADOW_FLAG) ;
  }

  /*
   * Link the object to the frame if it's not a CSG union object,
   * if it's clipped or if bounding slabs aren't used.
   */

  if ((Object->oType != CSG_UNION_TYPE) || (Object->Clip != NULL) || (!opts.Use_Slabs))
  {
    Link(Object, &(Object->Sibling), &(Frame.Objects));

    return;
  }

  /*
   * [DB 8/94]
   *
   * The object is a CSG union object. It will be split if all siblings are
   * finite, i.e. the volume of the bounding box doesn't exceed a threshold.
   */

  /* NK phmap - added code so union is not split up if it is
                flagged for hi-density photon mapping...
            maybe we SHOULD split it anyways... do speed tests later */
  if(!((CSG *)Object)->do_split)
  {
    Link(Object, &(Object->Sibling), &(Frame.Objects));
    return;
  }

  if (Object->Bound != NULL)
  {
    /* Test if all siblings are finite. */

    finite = true;

    for (This_Sib = ((CSG *)Object)->Children; This_Sib != NULL; This_Sib = This_Sib->Sibling)
    {
      BOUNDS_VOLUME(Volume, This_Sib->BBox);

      if (Volume > BOUND_HUGE)
      {
        finite = false;

        break;
      }
    }

    /*
     * If the union has infinite children or splitting is not used and
     * the object is not a light group link the union to the frame.
     */

    if (((!finite) || !(opts.Options & SPLIT_UNION)) && ((Object->Type & LIGHT_GROUP_OBJECT) != LIGHT_GROUP_OBJECT))
    {
      if (finite)
      {
        Warning(0, "CSG union unnecessarily bounded.");
      }

      Link(Object, &(Object->Sibling), &(Frame.Objects));

      return;
    }

    Warning(0, "Bounded CSG union split.");
  }

  /* Link all siblings of a union to the frame. */

  for (This_Sib = ((CSG *)Object)->Children; This_Sib != NULL; This_Sib = Next_Sib)
  {
    /* Link_To_Frame() changes Sibling so save it */

    Next_Sib = This_Sib->Sibling;

    /* Sibling is no longer inside a CSG object. */

    This_Sib->Type &= ~IS_CHILD_OBJECT;

    Link_To_Frame (This_Sib);
  }

/*
  Object->Texture = NULL;
*/

  Object->Sibling = NULL;

  ((CSG *)Object)->Children = NULL;

  Destroy_Object (Object);
}

/* NK layers - 1999 June 10 - for backwards compatiblity with layered textures */
void Convert_Filter_To_Transmit(PIGMENT *Pigment)
{
  int i;
  BLEND_MAP *Map;

  if (Pigment==NULL) return;

  switch (Pigment->Type)
  {
    case PLAIN_PATTERN:
      Pigment->Colour[pTRANSM]+=Pigment->Colour[pFILTER];
      Pigment->Colour[pFILTER]=0;
      break;

    default:
      if (Pigment->Blend_Map != NULL)
      {
        Map = Pigment->Blend_Map;
        /* go through blend map */
        if ((Map->Type == PIGMENT_TYPE) || (Map->Type == DENSITY_TYPE))
        {
           for (i = 0; i < Map->Number_Of_Entries; i++)
           {
             Convert_Filter_To_Transmit(Map->Blend_Map_Entries[i].Vals.Pigment);
           }
        }
        else
        {
           for (i = 0; i < Map->Number_Of_Entries; i++)
           {
             Map->Blend_Map_Entries[i].Vals.Colour[pTRANSM]+=Map->Blend_Map_Entries[i].Vals.Colour[pFILTER];
             Map->Blend_Map_Entries[i].Vals.Colour[pFILTER]=0;
           }
        }

      }

      break;
  }
}
void Link (OBJECT *New_Object, OBJECT  **Field, OBJECT  **Old_Object_List)
{
  *Field = *Old_Object_List;
  *Old_Object_List = New_Object;
  }
/*****************************************************************************
*
* FUNCTION
*
* INPUT
*
* OUTPUT
*
* RETURNS
*
* AUTHOR
*
* DESCRIPTION
*
* CHANGES
*
******************************************************************************/

void Post_Process (OBJECT *Object,OBJECT  *Parent)
{
  DBL Volume;
  OBJECT *Sib;
  FINISH *Finish;

  if (Object == NULL)
  {
    return;
  }

  if (Object->Type & LT_SRC_UNION_OBJECT) 
  {
    for (Sib = ((CSG *)Object)->Children; Sib != NULL; Sib = Sib->Sibling)
    {
      Post_Process(Sib, Object);
    }
    return;
  }

  /* Promote texture etc. from parent to children. */

  if (Parent != NULL)
  {
    if (Object->Texture == NULL)
    {
      Object->Texture = Copy_Texture_Pointer(Parent->Texture);
      /* NK 1998 copy uv_mapping flag if and only if we copy the texture */
      if (Test_Flag(Parent, UV_FLAG))
        Set_Flag(Object, UV_FLAG);
      /* NK ---- */
    }
    if (Object->Interior_Texture == NULL)
    {
      Object->Interior_Texture = Copy_Texture_Pointer(Parent->Interior_Texture);
      if(Test_Flag(Parent, UV_FLAG))
        Set_Flag(Object, UV_FLAG);
    }
    if (Object->Interior == NULL)
    {
      Object->Interior = Copy_Interior_Pointer(Parent->Interior);
    }

    if (Test_Flag(Parent, NO_REFLECTION_FLAG))
    {
      Set_Flag(Object, NO_REFLECTION_FLAG);
    }
    if (Test_Flag(Parent, NO_IMAGE_FLAG))
    {
      Set_Flag(Object, NO_IMAGE_FLAG);
    }
    if (Test_Flag(Parent, NO_SHADOW_FLAG))
    {
      Set_Flag(Object, NO_SHADOW_FLAG);
    }
    if (Test_Flag(Parent, MULTITEXTURE_FLAG))
    {
      Set_Flag(Object, MULTITEXTURE_FLAG);
    }

    /* NK phmap */
    /* promote photon mapping flags to child */
    if (Test_Flag(Parent, PH_TARGET_FLAG))
    {
      Set_Flag(Object, PH_TARGET_FLAG);
      Object->Ph_Density = Parent->Ph_Density;
      CheckPassThru(Object, PH_TARGET_FLAG);
    }

    if(Test_Flag(Parent, PH_PASSTHRU_FLAG))
    {
      Set_Flag(Object, PH_PASSTHRU_FLAG);
      CheckPassThru(Object, PH_PASSTHRU_FLAG);
    }

    if (Test_Flag(Parent, PH_RFL_ON_FLAG))
    {
      Set_Flag(Object, PH_RFL_ON_FLAG);
      Clear_Flag(Object, PH_RFL_OFF_FLAG);
    }
    else if (Test_Flag(Parent, PH_RFL_OFF_FLAG))
    {
      Set_Flag(Object, PH_RFL_OFF_FLAG);
      Clear_Flag(Object, PH_RFL_ON_FLAG);
    }

    if (Test_Flag(Parent, PH_RFR_ON_FLAG))
    {
      Set_Flag(Object, PH_RFR_ON_FLAG);
      Clear_Flag(Object, PH_RFR_OFF_FLAG);
      CheckPassThru(Object, PH_RFR_ON_FLAG);
    }
    else if (Test_Flag(Parent, PH_RFR_OFF_FLAG))
    {
      Set_Flag(Object, PH_RFR_OFF_FLAG);
      Clear_Flag(Object, PH_RFR_ON_FLAG);
    }

    if(Test_Flag(Parent, PH_IGNORE_PHOTONS_FLAG))
    {
      Set_Flag(Object, PH_IGNORE_PHOTONS_FLAG);
    }
  }

  if (Object->Interior != NULL)
  {
     Post_Media(Object->Interior->IMedia);
  }

  if ((Object->Texture == NULL) &&
      !(Object->Type & TEXTURED_OBJECT) &&
      !(Object->Type & LIGHT_SOURCE_OBJECT))
  {
    if (Parent)
    {
      if(Parent->oType != CSG_INTERSECTION_TYPE ||
         !Test_Flag(Parent, MULTITEXTURE_FLAG))
      {
		  TEXTURE *tex1 =new TEXTURE();//Create_Texture();
		  
		  tex1->Pigment = new PIGMENT();//Create_Pigment();
		  tex1->Tnormal = NULL;
		  tex1->Finish  = new FINISH (); //Create_Finish();
		  Object->Texture = tex1->GetCopy();//new TEXTURE(tex1);//Copy_Textures(tex1);
		  Destroy_Textures(tex1);
		//  delete tex1;
      }
    }
    else
	{
		TEXTURE *tex1 =new TEXTURE();//Create_Texture();
		  
		  tex1->Pigment = new PIGMENT();//Create_Pigment();
		  tex1->Tnormal = NULL;
		  tex1->Finish  =new FINISH ();// Create_Finish();
		  Object->Texture = tex1->GetCopy();//new TEXTURE(tex1);//Copy_Textures(tex1);
		Destroy_Textures(tex1);
	//	  delete tex1;
	}
  }

  if(!(Object->Type & LIGHT_GROUP_OBJECT) &&
     !(Object->Type & LIGHT_GROUP_LIGHT_OBJECT))
  {
    Post_Textures(Object->Texture);  /*moved cey 6/97*/

    if (Object->Interior_Texture)
    {
      Post_Textures(Object->Interior_Texture);
    }
  }

  if(Object->Type & LIGHT_SOURCE_OBJECT)
  {
    DBL len1,len2;
    LIGHT_SOURCE *Light = (LIGHT_SOURCE *)Object;


    /* check some properties of the orient light sources */
    if (Light->Orient)
    {
      if(!Light->Circular)
      {
        Light->Circular = true;
        Warning(0,"Orient can only be used with circular area lights. This area light is now circular.");
      }

      VLength(len1,Light->Axis1);
      VLength(len2,Light->Axis2);

      if(fabs(len1-len2)>EPSILON)
      {
        Warning(0, "When using orient, the two axes of the area light must be of equal length.\nOnly the length of the first axis will be used.");
        /*
        the equalization is actually done in the lighting code, since only the length of
        Axis1 will be used
        */
      }

      if(Light->Area_Size1 != Light->Area_Size2)
      {
        Warning(0, "When using orient, the two sample sizes for the area light should be equal.");
      }
    }

    /* Make sure that circular light sources are larger than 1 by x [ENB 9/97] */
    if (Light->Circular)
    {
      if ((Light->Area_Size1 <= 1) || (Light->Area_Size2 <= 1))
      {
        Warning(0, "Circular area lights must have more than one sample along each axis.");
        Light->Circular = false;
      }
    }
  }

  if (Object->Type & LIGHT_SOURCE_OBJECT)
  {
    // post-process the light source
    if (((LIGHT_SOURCE *)Object)->Projected_Through_Object != NULL) 
    {
      if (((LIGHT_SOURCE *)Object)->Projected_Through_Object->Interior != NULL)
      {
         Destroy_Interior(((LIGHT_SOURCE *)Object)->Projected_Through_Object->Interior);
         ((LIGHT_SOURCE *)Object)->Projected_Through_Object->Interior=NULL;
         Warning(0,"Projected through objects can not have interior, interior removed.");
      }
      if (((LIGHT_SOURCE *)Object)->Projected_Through_Object->Texture != NULL) 
      {
         Destroy_Textures(((LIGHT_SOURCE *)Object)->Projected_Through_Object->Texture);
//  delete ((LIGHT_SOURCE *)Object)->Projected_Through_Object->Texture;
		  ((LIGHT_SOURCE *)Object)->Projected_Through_Object->Texture = NULL;
         Warning(0,"Projected through objects can not have texture, texture removed.");
      }
    }

    // only global light sources are in Frame.Light_Sources list [trf]
    if(!(Object->Type & LIGHT_GROUP_LIGHT_OBJECT))
    {
      // add this light to the frame's list of global light sources
      ((LIGHT_SOURCE *)Object)->Next_Light_Source = Frame.Light_Sources;

      Frame.Light_Sources = (LIGHT_SOURCE *)Object;

      Frame.Number_Of_Light_Sources++;
    }
    else
    {
      // Put it into the frame's list of light-source lights

      // NOTE - We use LIGHT_GROUP_LIGHT to create this linked list, because we can't
      //        use the Next_Light_Source pointer that is part of the light source.
      //        We can't use that pointer because it is used for light group purposes,
      //        and here we just want a list of all lights that are in any light group.

      LIGHT_GROUP_LIGHT *Node = (LIGHT_GROUP_LIGHT*)POV_MALLOC(sizeof(LIGHT_GROUP_LIGHT),"light_group_light");
      
      Node->Light = (LIGHT_SOURCE *)Object;

      // link node to list
      Node->Next = Frame.Light_Group_Lights;
      Frame.Light_Group_Lights = Node;
    }
  }
  else
  {
    // post-process the object

    /* If there is no interior create one. */

    if (Object->Interior == NULL)
    {
      Object->Interior = Create_Interior();
    }

    /* Promote hollow flag to interior. */

    Object->Interior->hollow = (Test_Flag(Object, HOLLOW_FLAG) != false);

    /* Promote finish's IOR to interior IOR. */

    if (Object->Texture != NULL)
    {
      if (Object->Texture->Type == PLAIN_PATTERN)
      {
        if ((Finish = Object->Texture->Finish) != NULL)
        {
          if (Finish->Temp_IOR >= 0.0)
          {
            Object->Interior->IOR = Finish->Temp_IOR;
            Object->Interior->Dispersion = Finish->Temp_Dispersion;
          }
          if (Finish->Temp_Caustics >= 0.0)
          {
            Object->Interior->Caustics = Finish->Temp_Caustics;
          }

          Object->Interior->Old_Refract = Finish->Temp_Refract;
        }
      }
    }

    /* If there is no IOR specified use the atmopshere ior. */

    if (Object->Interior->IOR == 0.0)
    {
      Object->Interior->IOR = Frame.Atmosphere_IOR;
      Object->Interior->Dispersion = Frame.Atmosphere_Dispersion;
    }
  }

  if (Object->Type & IS_COMPOUND_OBJECT)
  {
    for (Sib = ((CSG *)Object)->Children; Sib != NULL; Sib = Sib->Sibling)
    {
      Post_Process(Sib, Object);
    }
  }
  /* Test wether the object is finite or infinite. [DB 9/94] */

  BOUNDS_VOLUME(Volume, Object->BBox);

  if (Volume > BOUND_HUGE)
  {
    Set_Flag(Object, INFINITE_FLAG);
  }

  /* Test wether the object is opaque or not. [DB 8/94] */

  if (//(Object->Methods != &Blob_Methods) &&
      (Object->oType != MESH_TYPE) &&
      (Test_Opacity(Object->Texture)) && 
      ((Object->Interior_Texture==NULL) || 
        Test_Opacity(Object->Interior_Texture)))
  {
    Set_Flag(Object, OPAQUE_FLAG);
  }
  else
  {
    /* Objects with multiple textures have to be handled separately. */

  /*  if (Object->Methods == &Blob_Methods)
    {
      Test_Blob_Opacity((BLOB *)Object);
    }
*/
    if (Object->oType == MESH_TYPE)
    {
      Test_Mesh_Opacity((MESH *)Object);
    }
  }
}
void Destroy_Frame()
{
    FOG *Fog, *Next_Fog;
    RAINBOW *Rainbow, *Next_Rainbow;

    // This is necessary as a user who hits CANCEL during any IO performed
    // by this routine (e.g. Destroy_Object(), which can complain about
    // isosurface max_gradient), will cause this routine to be entered again
    // before the relevent data member has been set to NULL (this is able
    // to happen since cancel will invoke a longjmp on most platforms).
    // This causes the currently-executing segment to be destroyed twice,
    // which is a Bad Thing(tm). [CJC 11/01]
/*    if (Destroying_Frame)
      return ;
    Destroying_Frame = true ;
*/
    delete Frame.Camera;
    Frame.Camera=NULL;

    /* Destroy fogs. [DB 12/94] */

    for (Fog = Frame.Fog; Fog != NULL;)
    {
      Next_Fog = Fog->Next;

//      Destroy_Fog(Fog);
	  delete Fog;

      Fog = Next_Fog;
    }
    
    Frame.Fog = NULL;

    /* Destroy rainbows. [DB 12/94] */

    for (Rainbow = Frame.Rainbow; Rainbow != NULL;)
    {
      Next_Rainbow = Rainbow->Next;

//      Destroy_Rainbow(Rainbow);
	  delete Rainbow;

      Rainbow = Next_Rainbow;
    }

    Frame.Rainbow = NULL;

    /* Destroy skysphere. [DB 12/94] */

    delete Frame.Skysphere;

    Frame.Skysphere = NULL;

    /* Destroy atmosphere. [DB 1/95] */

    Destroy_Media(Frame.Atmosphere);

    Frame.Atmosphere = NULL;

    if (Frame.Objects != NULL) {
       Destroy_Object (Frame.Objects);
       Frame.Objects = NULL;
       Frame.Light_Sources = NULL;
    }

    if(Frame.Light_Group_Lights != NULL)
    {
      LIGHT_GROUP_LIGHT* Light_Group_Light;
      LIGHT_GROUP_LIGHT* Next;
      for (Light_Group_Light = Frame.Light_Group_Lights;
           Light_Group_Light != NULL; )
      {
        Next = Light_Group_Light->Next;
        POV_FREE(Light_Group_Light);
        Light_Group_Light = Next;
      }

      Frame.Light_Group_Lights = NULL;
    }
}